@@ -1,15 +1,17 @@
 """
 The main class which you import and use as a decorator.
 """
+import datetime
 import functools
 import json
 import logging
 import re
 import sys
 import traceback
+from decimal import Decimal
 from http import HTTPStatus
 from inspect import isclass, signature
-from typing import Iterable, Optional, Union
+from typing import Any, Iterable, Optional, Union
 
 from awslambdaric.lambda_context import LambdaContext
 from orjson import loads
@@ -19,6 +21,8 @@ from pydantic_lambda_handler.middleware import BaseHook
 from pydantic_lambda_handler.models import BaseOutput
 from pydantic_lambda_handler.params import Header, Param, Query
 
+JSON_DATA_TYPES = int, str, float, Decimal, bool, datetime.datetime, datetime.date
+
 
 class PydanticLambdaHandler:
     """
@@ -50,11 +54,12 @@ class PydanticLambdaHandler:
         url,
         *,
         status_code: Union[HTTPStatus, int] = HTTPStatus.OK,
-        operation_id: str = None,
+        operation_id: Optional[str] = None,
         description: str = "Successful Response",
         function_name=None,
         response_model=None,
         logger=None,
+        errors: Optional[list[tuple[Union[HTTPStatus, int], Any]]] = None,
     ):
         """Expect request with a GET method.
 
@@ -66,13 +71,7 @@ class PydanticLambdaHandler:
         if logger:
             self.logger = logger
         return self.run_method(
-            method,
-            url,
-            status_code,
-            operation_id,
-            description,
-            function_name,
-            response_model,
+            method, url, status_code, operation_id, description, function_name, response_model, errors
         )
 
     def post(
@@ -80,11 +79,12 @@ class PydanticLambdaHandler:
         url,
         *,
         status_code: Union[HTTPStatus, int] = HTTPStatus.CREATED,
-        operation_id: str = None,
+        operation_id: Optional[str] = None,
         description: str = "Successful Response",
         function_name=None,
         response_model=None,
         logger=None,
+        errors=None,
     ):
         """Expect request with a POST method.
 
@@ -103,9 +103,10 @@ class PydanticLambdaHandler:
             description,
             function_name,
             response_model,
+            errors,
         )
 
-    def run_method(
+    def run_method(  # noqa: C901 too complex
         self,
         method,
         url,
@@ -114,6 +115,7 @@ class PydanticLambdaHandler:
         description,
         function_name,
         response_model,
+        errors,
     ):
         for hook in self._hooks:
             hook.method_init(**locals())
@@ -167,9 +169,34 @@ class PydanticLambdaHandler:
                             None,
                         ):
                             func_kwargs[context_name] = context
-                        body = func(**func_kwargs)
                     else:
-                        body = func()
+                        func_kwargs = {}
+
+                    try:
+                        body = func(**func_kwargs)
+                    except Exception as e:
+                        if errors:
+                            for e_status_code, exceptions in errors:
+                                if isinstance(e, exceptions):
+                                    if hasattr(e, "json"):
+                                        body = json.dumps({"detail": json.loads(e.json())})
+                                    else:
+                                        body = json.dumps(
+                                            {
+                                                "detail": [
+                                                    {
+                                                        "msg": getattr(e, "msg", str(e)),
+                                                        "type": type(e).__name__,
+                                                    }
+                                                ]
+                                            }
+                                        )
+                                    response = BaseOutput(
+                                        body=body,
+                                        status_code=e_status_code,
+                                    )
+                                    return loads(response.json())
+                        raise
 
                     for hook in reversed(self._hooks):
                         body = hook.post_func(body)
@@ -247,16 +274,14 @@ class PydanticLambdaHandler:
                     body_model._alias = param
                 elif isclass(param_info.annotation) and issubclass(model, LambdaContext):
                     additional_kwargs[param] = annotations
+                elif isclass(annotations[0]) and issubclass(annotations[0], (JSON_DATA_TYPES)):
+                    query_model_dict[param] = annotations
+                elif annotations[0].__args__[0].__name__ == "list":
+                    multiquery_model_dict[param] = annotations
                 else:
-                    # FIXME: This will need some refactoring
-                    if not isclass(annotations[0]) or not issubclass(annotations[0], (int, str)):
-                        if annotations[0].__args__[0].__name__ == "list":
-                            multiquery_model_dict[param] = annotations
-                        else:
-                            raise ValueError("Something went wrong")
-                    else:
-                        query_model_dict[param] = annotations
+                    query_model_dict[param] = annotations
 
+                model, body_default = annotations
         if path_parameters != set(path_model_dict.keys()):
             raise ValueError("Missing path parameters")
 
